;VERY SMALL 4 DIGIT LED CLOCK WITH 50HZ MAINS FREQUENCY CLOCK SOURCE

;ATTiny24 @ 1 MHz internal R/C Oscillator
;Fuse Bytes: low = 0x42, high = 0xDD (Brown-out at 2.7 V)

;Ports:
;  PA0 = Display 4 (right)
;  PA1 = Display 3
;  PA2 = Display 2
;  PA3 = Display 1 (left)
;  PA4 = 100 Hz input (50 Hz mains --> rectifier --> 100 Hz)
;  PA5 = 'Display On/Off' push button (to ground)
;  PA6 = Display Segments Shift Register Data
;  PA7 = Display Segments Shift Register Clock
;  PB1 = 'Set Hours' pushbutton (to ground)
;  PB2 = 'Set Minutes' pushbutton (to ground)

.include "../include/tn24def.inc"

.def	temp = R16
.def	temp2 = R17
.def	display = R18
.def	hours = R19
.def	minutes = R20
.def	displayon = R21
.def	hourkey = R22
.def	minutekey = R23
.def	onoffkey = R24
.def	inputsig = R25
.def	countL = R26
.def	countH = R27

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org OVF0addr
	rjmp	multiplex
	
;===============================================================================

multiplex:
	;multiplex displays (~488 Hz -> ~122 Hz refresh rate)
	push	temp
	push	temp2
	in	temp, SREG
	push	temp2
	
	;capture 100 Hz clock signal
	in	temp, PINA
	andi	temp, 0x10
	cp	temp, inputsig
	breq	input_end
	mov	inputsig, temp
	cpi	inputsig, 0
	brne	input_end
	adiw	countH:countL, 1
	ldi	temp, HIGH(6000)
	cpi	countL, LOW(6000)
	cpc	countH, temp
	brlo	input_end
	rcall	inc_minutes
	ldi	countL, 0
	ldi	countH, 0
input_end:
	
	;select bit mask and data for next digit
	lsr	display
	brne	PC+2
	ldi	display, 0x08
	mov	temp2, minutes ;display 4
	sbrc	display, 0
	mov	temp, temp2
	swap	temp2 ;display 3
	sbrc	display, 1
	mov	temp, temp2
	mov	temp2, hours ;display 2
	sbrc	display, 2
	mov	temp, temp2
	swap	temp2 ;display 1
	sbrc	display, 3
	mov	temp, temp2
	
	;convert data to 7-segment
	ldi	ZL, LOW(segments*2)
	ldi	ZH, HIGH(segments*2)
	andi	temp, 0x0F
	ldi	temp2, 0
	add	ZL, temp
	add	ZH, temp2
	lpm	temp, Z
;	sbrc	display, 2 ;add decimal point for display 2
;	ori	temp, 0x20
	
	;display off
	ldi	temp2, 0x2F
	out	PORTA, temp2
	
	;send data to shift register
	ldi	temp2, 8
multiplex_shift:
	sbi	PORTA, 6
	lsr	temp
	brcc	PC+2 ;invert segments!
	cbi	PORTA, 6
	sbi	PORTA, 7
	cbi	PORTA, 7
	dec	temp2
	brne	multiplex_shift
	
	;display on
	cpi	displayon, 0
	breq	multiplex_end
	mov	temp, display
	com	temp
	andi	temp, 0x2F
	out	PORTA, temp
	
multiplex_end:
	;return
	pop	temp
	out	SREG, temp
	pop	temp2
	pop	temp
	reti
	
segments:
	;segment order: MSB -> g, c, dp, d, a, f, e, b <- LSB
	;LSB sent first -> appears on shift register output Q7
	;     0     1     2     3     4     5     6     7     8     9
	.db 0x5f, 0x41, 0x9b, 0xd9, 0xc5, 0xdc, 0xde, 0x49, 0xdf, 0xdd
	
;===============================================================================

reset:
	;set stackpointer
	ldi	temp, RAMEND
	out	SPL, temp
	
	;init ports
	ldi	temp, 0xCF
	out	DDRA, temp
	ldi	temp, 0x2F
	out	PORTA, temp
	ldi	temp, 0x00
	out	DDRB, temp
	ldi	temp, 0x07
	out	PORTB, temp
	
	;setup Timer/Counter 0 interrupt (multiplexing)
	ldi	temp, 0x02 ;Clk / 8
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0
	out	TIMSK0, temp
	
	;init registers
	ldi	display, 0x08 ;bit mask
	ldi	hours, 0x00
	ldi	minutes, 0x00
	ldi	displayon, 1
	ldi	hourkey, 0
	ldi	minutekey, 0
	ldi	onoffkey, 0
	ldi	inputsig, 0
	ldi	countH, 0
	ldi	countL, 0
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------

loop:
	;"Display on/off" key
 	sbis	PINA, 5
	rjmp	disponoff_pressed
	ldi	onoffkey, 0
	rjmp	disponoff_end
disponoff_pressed:
	inc	onoffkey
	cpi	onoffkey, 3
	brlo	disponoff_end
	ldi	temp, 1
	brne	PC+2
	eor	displayon, temp
	ldi	onoffkey, 5
disponoff_end:

	;"Set Hours" key
 	sbis	PINB, 1
	rjmp	sethour_pressed
	ldi	hourkey, 0
	rjmp	sethour_end
sethour_pressed:
	inc	hourkey
	cpi	hourkey, 3
	brlo	sethour_end
	brne	PC+2
	rcall	inc_hours
	cpi	hourkey, 200
	brlo	PC+3
	rcall	inc_hours
	ldi	hourkey, 150
sethour_end:

	;"Set Minutes" key
	sbis	PINB, 2
	rjmp	setminute_pressed
	ldi	minutekey, 0
	rjmp	setminute_end
setminute_pressed:
	inc	minutekey
	cpi	minutekey, 3
	brlo	setminute_end
	brne	setminute_continue
	ldi	countH, 0
	ldi	countL, 0
	rcall	inc_minutes
	rjmp	setminute_end
setminute_continue:
	cpi	minutekey, 200
	brlo	setminute_end
	ldi	countH, 0
	ldi	countL, 0
	rcall	inc_minutes
	ldi	minutekey, 150
setminute_end:

	;short delay
	ldi	temp, 5
	clr	temp2
delay:
	dec	temp2
	brne	delay
	dec	temp
	brne	delay

	rjmp	loop

;===============================================================================

inc_minutes:
	;increment minutes
	subi	minutes, -7
	brhc	PC+2
	subi	minutes, 6
	cpi	minutes, 0x60
	brsh	PC+2
	ret
	ldi	minutes, 0x00
	
inc_hours:
	;increment hours
	subi	hours, -7
	brhc	PC+2
	subi	hours, 6
	cpi	hours, 0x24
	brlo	PC+2
	ldi	hours, 0x00
	ret
	